#include "BaseTab.hpp"
#include "AppendGameDialog.hpp"
#include <wx/filename.h>

BaseTab::BaseTab(wxFrame *parent, std::string base_file)
    : TabBase(parent), TabInfos(TabInfos::BASE), base_file(base_file){
  
  // First open the database
  OpenDatabase(base_file);

  // Games tab
  games_tab=new BaseGameTab((wxFrame *)notebook,base,this);
  glm=games_tab->glm;
  notebook->AddPage(games_tab, "Games list",true); // true for selecting the tab
  // Import tab
  import_tab=new BaseImportTab((wxFrame *)notebook,base,this);
  notebook->AddPage(import_tab, "Import games");
  // Manage tab
  manage_tab=new BaseManageTab((wxFrame *)notebook, base, games_tab->glm);
  notebook->AddPage(manage_tab, "Manage database");

  // Refresh dynamic elements of the database (tab title, available db for import etc.)
  Refresh();

  // Bindings
  this->Bind(OPEN_GAME_EVENT, &BaseTab::OnOpenGame, this, wxID_ANY);
  this->Bind(wxEVT_BUTTON, &BaseTab::OnSave, this, ID_SAVE_BUTTON);
}


void BaseTab::OnOpenGame(wxCommandEvent &event){
  std::shared_ptr<Game> *g = (std::shared_ptr<Game>*)event.GetClientData();
  this->game=*g;

  // Ask MainFrame to open a new game
  // TODO: Simplify that is, use wxWidget main app to do it
  wxCommandEvent newGameEvent(NEW_GAME_EVENT, GetId());
  newGameEvent.SetEventObject(this);
  newGameEvent.SetClientData((TabInfos*)this);
  ProcessEvent(newGameEvent);
}

void BaseTab::Refresh(){
  import_tab->RefreshImportLists();
  SetLabel(wxFileName(base->GetFilePath()).GetName()+" [DB]"); // Propagated to MainWindow tab title automatically by wxWidget
}

void BaseTab::OpenDatabase(std::string dbpath) {
  wxFileName file(dbpath);
  wxString ext = file.GetExt().Lower();
  wxLogDebug("Here");
  if (ext == "pgn") {
    if(!file.Exists())
      PGNGameBase::CreateDatabaseFile(dbpath);
    base.reset();
    base = std::shared_ptr<GameBase>(new PGNGameBase(dbpath));
  }
}

void BaseTab::OnSave(wxCommandEvent &event) {
  // Build new games
  std::vector<std::shared_ptr<Game>> new_games=games_tab->GetEditedGames();
  for(auto g: import_tab->GetGameToImport()){
    new_games.push_back(g);
  }
  SHOW_DIALOG_BUSY("Apply all changes. Take a coffee, this process can takes time...");
  base->Save(games_tab->GetDeletedGameIds(), import_tab->GetDatabaseToImport(), new_games);
  
  // Close all opened games in this database
  wxCommandEvent closeLinkedTabEvent(CLOSE_LINKED_TAB, GetId());
  closeLinkedTabEvent.SetClientData((TabInfos*)this);
  ProcessEvent(closeLinkedTabEvent);

  // Reopen the saved database
  OpenDatabase(base_file);
  games_tab->Reset(base);
  manage_tab->Reset(base);
  import_tab->Reset(base);
}